from datetime import datetime
from typing import Any, Optional

from .base import AbstractApi


class ArchiveV1(AbstractApi):
    _api_path = "archive/v1"

    def search(
        self,
        *,
        intersects: Optional[Any] = None,
        date_from: Optional[datetime] = None,
        date_to: Optional[datetime] = None,
        limit=25,
        page_token: Optional[str] = None,
        **kwargs,
    ):
        url = self._url("/search")
        payload = {
            **kwargs,
            "intersects": intersects,
            "limit": limit,
            "token": page_token,
            "datetime": "/".join(
                filter(
                    lambda x: x,
                    [
                        date_from and date_from.isoformat(),
                        date_to and date_to.isoformat(),
                    ],
                )
            ),
        }

        return self.client.post(
            url,
            json={k: v for k, v in payload.items() if v},
        )



from abc import ABC
from urllib.parse import urljoin

from satellitevu.http import AbstractClient


class AbstractApi(ABC):
    client: AbstractClient
    base_url: str
    api_path: str

    def __init__(self, client: AbstractClient, base_url: str):
        self.client = client
        self.base_url = base_url

    def _url(self, path: str) -> str:
        api_base_url = urljoin(self.base_url, self._api_path.lstrip("/"))
        if api_base_url[-1] != "/":
            api_base_url += "/"
        return urljoin(api_base_url, path.lstrip("/"))



from typing import Optional, Union

from satellitevu.apis.archive import ArchiveV1
from satellitevu.apis.orders import OrdersV1
from satellitevu.auth import AbstractCache, Auth
from satellitevu.config import GATEWAY
from satellitevu.http import AbstractClient, UrllibClient


class Client:
    _client: AbstractClient
    _gateway_url: str

    auth: Auth

    archive_v1: ArchiveV1
    orders_v1: OrdersV1

    def __init__(
        self,
        *,
        client_id: str,
        client_secret: str,
        audience: Optional[str] = None,
        cache: Optional[AbstractCache] = None,
        auth_url: Optional[str] = None,
        http_client: Optional[AbstractClient] = None,
        gateway_url: Optional[str] = None,
    ):
        self._gateway_url = gateway_url or GATEWAY
        self._client = http_client or self._setup_client()

        self.auth = Auth(
            client_id=client_id,
            client_secret=client_secret,
            audience=audience,
            cache=cache,
            auth_url=auth_url,
            client=self._client,
        )
        self._client.set_auth(self._gateway_url, self.auth)

        self.archive_v1 = ArchiveV1(self._client, self._gateway_url)
        self.orders_v1 = OrdersV1(self._client, self._gateway_url)

    def _setup_client(self) -> AbstractClient:
        client = self._setup_requests_session()
        if client is None:
            client = UrllibClient()
        return client

    def _setup_requests_session(self) -> Union[AbstractClient, None]:
        try:
            from requests import Session

            from satellitevu.http.requests import RequestsSession
        except ImportError:
            return

        # TODO: Retries
        # TODO: Timeout
        session = Session()
        client = RequestsSession(instance=session)
        return client



from datetime import datetime
from json import dumps
from urllib.parse import urlparse

from mocket import Mocket, mocketize
from mocket.mockhttp import Entry
from pytest import mark


@mocketize(strict_mode=True)
@mark.parametrize(
    "kwargs, payload",
    (
        ({}, {"limit": 25}),
        ({"limit": 50}, {"limit": 50}),
        (
            {
                "date_from": datetime(2022, 9, 10, 0, 0, 0),
                "date_to": datetime(2022, 10, 10, 0, 0, 0),
            },
            {"limit": 25, "datetime": "2022-09-10T00:00:00/2022-10-10T00:00:00"},
        ),
        ({"bbox": [0, 0, 1, 1]}, {"bbox": [0, 0, 1, 1], "limit": 25}),
    ),
)
def test_search(client, oauth_token_entry, kwargs, payload):
    Entry.single_register(
        "POST", client._gateway_url + "archive/v1/search", "mock-stac-response"
    )

    response = client.archive_v1.search(**kwargs)

    requests = Mocket.request_list()
    assert len(requests) == 2

    api_request = requests[-1]
    assert api_request.headers["Host"] == urlparse(client._gateway_url).hostname
    assert api_request.path == "/archive/v1/search"
    assert api_request.headers["Content-Type"] == "application/json"
    assert api_request.headers["Authorization"] == "Bearer mock-token"
    assert api_request.body == dumps(payload)
    assert response.text == "mock-stac-response"



from abc import ABC, abstractmethod
from typing import Any, Dict, Optional


class ResponseWrapper(ABC):
    raw: Any
    headers: Dict[str, str]
    status: int
    json: Any
    text: str

    @abstractmethod
    def json(self):
        pass


class AbstractClient(ABC):
    """
    Abstract HTTP client
    """

    _auth: Dict[str, Any]

    def __init__(self):
        self._auth = {}

    def post(
        self,
        url,
        *,
        headers: Optional[Dict] = None,
        data: Optional[Dict] = None,
        json: Optional[Any] = None,
    ) -> ResponseWrapper:
        return self.request("POST", url, headers=headers, data=data, json=json)

    @abstractmethod
    def request(
        self,
        method: str,
        url: str,
        *,
        headers: Optional[Dict] = None,
        data: Optional[Dict] = None,
        json: Optional[Any] = None,
    ) -> ResponseWrapper:
        pass

    def set_auth(self, base_url: str, auth):
        self._auth[base_url] = auth

    def _set_auth(self, url: str, headers):
        auth = next((v for k, v in self._auth.items() if url.startswith(k)), None)
        has_auth = next(
            (True for k in headers.keys() if k.lower() == "authorization"), False
        )
        if auth and not has_auth:
            headers["authorization"] = f"Bearer {auth.token()}"



from importlib import import_module
from json import dumps
from unittest.mock import Mock

from mocket import Mocket, Mocketizer
from mocket.mockhttp import Entry
from pytest import fixture, mark, param, skip

from satellitevu.auth.auth import Auth

from . import ResponseWrapper, UrllibClient

try:
    from requests import Session as RequestsSession
except ImportError:
    RequestsSession = None
try:
    from httpx import Client as HttpxClient
except ImportError:
    HttpxClient = None


@fixture(
    params=(
        param("UrllibClient"),
        param(
            "requests.RequestsSession",
            marks=[
                mark.skipif(RequestsSession is None, reason="requests is not installed")
            ],
        ),
        param(
            "httpx.HttpxClient",
            marks=[mark.skipif(HttpxClient is None, reason="httpx is not installed")],
        ),
    )
)
def http_client_class(request):
    full_path = f"{__package__}.{request.param}"
    module = import_module(full_path.rsplit(".", maxsplit=1)[0])
    return getattr(module, full_path.rsplit(".")[-1])


@mark.parametrize("method", ("GET", "POST"))
def test_http_client(http_client_class, method):
    client = http_client_class()

    Entry.single_register(
        method, "http://example.com/", body=dumps({"message": "Hello"})
    )
    with Mocketizer():
        response = client.request(method, "http://example.com/")

    assert isinstance(response, ResponseWrapper)
    assert response.json() == {"message": "Hello"}


def test_http_custom_actor(http_client_class):
    if http_client_class == UrllibClient:
        skip("UrllibClient does not support custom instance")

    instance = Mock()
    client = http_client_class(instance=instance)
    client.request("GET", "http://example.com/")

    assert instance.request.called


@mark.parametrize(
    "url, headers, uses_injected_auth",
    (
        ("http://example.com/", None, False),
        ("http://api.example.com/", None, False),
        ("http://api.example.com/non-authed", None, False),
        ("http://api.example.com/authed/", None, True),
        ("http://api.example.com/authed/subpath", None, True),
        (
            "http://api.example.com/authed/subpath",
            {"Authorization": "some-other"},
            False,
        ),
    ),
)
def test_http_set_auth(http_client_class, url, headers, uses_injected_auth):
    auth = Mock(wraps=Auth(client_id="mocked", client_secret="mocked"))
    auth.token.return_value = "mock-token"
    Entry.single_register("GET", url)

    client = http_client_class()
    client.set_auth("http://api.example.com/authed/", auth)

    with Mocketizer():
        client.request("GET", url, headers=headers)
        requests = Mocket.request_list()

    assert len(requests) == 1
    assert (
        requests[0].headers.get("Authorization") == "Bearer mock-token"
    ) == uses_injected_auth


@mark.parametrize(
    "data, json, body, content_type",
    (
        ({"foo": "bar"}, None, "foo=bar", "application/x-www-form-urlencoded"),
        (None, {"foo": "bar"}, '{"foo": "bar"}', "application/json"),
        (
            {"bar": "foo"},
            {"foo": "bar"},
            "bar=foo",
            "application/x-www-form-urlencoded",
        ),
    ),
)
def test_payload(http_client_class, data, json, body, content_type):
    client = http_client_class()
    Entry.single_register("POST", "http://api.example.com")

    with Mocketizer():
        client.request("POST", "http://api.example.com", data=data, json=json)
        request = Mocket.last_request()
    assert request.headers["Content-Type"] == content_type
    assert request.body == body



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['satellitevu', 'satellitevu.apis', 'satellitevu.auth', 'satellitevu.http']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0']

setup_kwargs = {
    'name': 'satellitevu',
    'version': '0.9.0',
    'description': "Client SDK for SatelliteVu's platform APIs",
    'long_description': '# SatelliteVu Platform API Client SDK\n\nLightweight API Client SDK for SatelliteVu\'s Platform APIs, providing authorization\nhandling and convenience methods to interact wit the published APIs.\n\n## Installation\n\nTBD\n\nFor now, need to use the repo as an git based dependency. Can use the `requests` or\n`httpx` optional dependency groups, as these http client implementations can be used\ninstead of the default urllib based client.\n\n## Usage\n\nA User API Client credential set consisting of an _client id_ and _client secret_ is\nneeded and should be set in your script\'s environment variables.\n\nCheck out the [examples](./examples/) provided. They can for example be run locally with\n\n```\npoetry run python ./examples/archive.py --example=recent\n```\n\n### Simple Client Usage\n\nThe easiest way to get started is to use the `satellitevu.Client` class, which needs\na client_id and client_secret only:\n\n```\nimport os\n\nfrom satellitevu import Client\n\n\nclient = Client(os.getenv("CLIENT_ID"), os.getenv("CLIENT_SECRET"))\nprint(client.archive_v1.search().json())\n```\n\n`client.archive_v1.search` supports all supported request body parameters documented\nin the [API docs](search-api-docs), with special handling for `datetime` which is\nconstructed from the optional `date_from` and `date_to` parameters and a default result\npage size limit of 25.\n\n### Authentication Handling\n\nThe `satellitevu.Auth` class provides the main interface to retrieve an\nauthorization token required to interact with the API endpoints.\n\n```\nimport os\n\nfrom satellitevu import Auth\n\n\nauth = Auth(os.getenv("CLIENT_ID"), os.getenv("CLIENT_SECRET"))\nprint(auth.token())\n```\n\nThus retrieved token can be used for bearer token authentication in HTTP request\nAuthorization headers.\n\nThe `Auth` class by default uses a file based cache which will store the token in\n\n- `~/.cache/SatelliteVu` on Linux\n- `~/Library/Caches/SatelliteVu` on MacOS\n- `C:\\Documents and Settings\\<username>\\Local Settings\\Application Data\\SatelliteVu\\Cache`\n  on Windows\n\nOther cache implementations must implement the `satellitevu.auth.cache.AbstractCache`\nclass.\n\n### HTTP Client Wrappers\n\nConvenience wrapper classes for common HTTP client implementations are provided as\nimplementations of `satellitevu.http.AbstractClient`, which provides an `request` method\nwith an interface similar to `requests.request` and returning an\n`satellitevu.http.ResponseWrapper` instance, where the response object of the underlying\nimplementation is available in the `raw` property.\n\nCommonly used properties and methods are exposed on both `AbstractClient` and\n`ResponseWrapper`.\n\n- `satellitevu.http.UrllibClient` for Python standard lib\'s `urllib`\n- `satellitevu.http.requests.RequestsSession` using `requests.Session` class\n- `satellitevu.http.httpx.HttpxClient` using `httpx.Client` (Todo)\n\nImplementations based on `requests` and `httpx` allow setting an instance of the\nunderlying implementation, but will provide a default instance if not.\n\n## Developer Setup\n\n### Requirements\n\n- Installations of Python 3.8 and 3.10 (for example using [Pyenv][pyenv])\n- Global installations (for example managed with [pipx][pipx]) of\n  - Python [Poetry](poetry)\n  - Python [nox](nox) with [nox-poetry](nox-poetry) plugin\n- Make (optional)\n\nExample global setup (with `pyenv` and `pipx`):\n\n```\npyenv install 3.8.13\npyenv install 3.10.4\npipx install poetry\npipx install nox\npipx inject nox nox-poetry\n```\n\nPlease bootstrap your environment with `make bootstrap`.\n\n#### Example test setup with `pyenv` and `pipx`\n\nRun tests against Python version used by poetry:\n\n```\nmake test\nmake PYTEST_ADDOPTS="-x -s" test\n```\n\nor\n\n```\npoetry run pytest\nPYTEST_ADDOPTS="-x -s" poetry run pytest\n```\n\nRun tests against Python 3.8 and Python 3.10:\n\n```\npyenv shell 3.8.13 3.10.4\nnox\n```\n\n[pyenv]: https://github.com/pyenv/pyenv\n[poetry]: https://python-poetry.org\n[pipx]: https://pypa.github.io/pipx/\n[nox]: https://nox.thea.codes/en/stable/\n[nox-poetry]: https://nox-poetry.readthedocs.io/en/stable/\n[search-api-docs]: https://api.satellitevu.com/archive/v1/docs#operation/Search_search_post\n',
    'author': 'Christian Wygoda',
    'author_email': 'christian.wygoda@satellitevu.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from json import dumps
from typing import Dict, Optional
from urllib.parse import urljoin

from mocket.mockhttp import Entry
from pytest import fixture

from satellitevu.client import Client

from .auth.cache import AbstractCache


class MemoryCache(AbstractCache):
    cache: Dict[str, str]

    def __init__(self):
        super().__init__()
        self.cache = {}

    def save(self, client_id: str, value: str):
        self.cache[client_id] = value

    def load(self, client_id: str) -> Optional[str]:
        return self.cache.get(client_id)


@fixture
def client(memory_cache):
    return Client(client_id="mock-id", client_secret="mock-secret", cache=memory_cache)


@fixture
def memory_cache():
    return MemoryCache()


@fixture
def oauth_token_entry(client):
    return Entry.single_register(
        "POST",
        urljoin(client.auth.auth_url, "oauth/token"),
        body=dumps({"access_token": "mock-token"}),
    )


@fixture
def redirect_response():
    return {"url": "https://image.test"}


@fixture
def order_details_response():
    return {
        "id": "528b0f77-5df1-4ed7-9224-502817170613",
        "type": "FeatureCollection",
        "features": [
            {
                "id": "8566d747-8833-44a7-a1c8-8d13e541f673",
                "properties": {
                    "item_id": "image",
                    "owned_by": "random",
                    "created_at": "2022-10-20T09:23:14.959823",
                    "updated_at": "2022-10-20T09:23:14.959823",
                },
            }
        ],
    }



from ast import Dict
from typing import Any, Optional

from httpx import Client, Response

from .base import AbstractClient
from .base import ResponseWrapper as BaseResponse


class ResponseWrapper(BaseResponse):
    raw: Response

    def __init__(self, raw: Response):
        self.raw = raw
        self.status = raw.status_code
        self.headers = raw.headers

    def json(self):
        return self.raw.json()

    @property
    def text(self):
        return self.raw.text


class HttpxClient(AbstractClient):
    client: Client

    def __init__(self, instance: Optional[Client] = None):
        super().__init__()
        self.client = instance or Client()

    def request(
        self,
        method: str,
        url: str,
        *,
        headers: Optional[Dict] = None,
        data: Optional[Dict] = None,
        json: Optional[Any] = None
    ) -> ResponseWrapper:
        headers = headers or {}

        self._set_auth(url, headers)
        response = self.client.request(
            method=method, url=url, headers=headers, data=data, json=json
        )
        return ResponseWrapper(response)
